/* ===============================================
 * Author: Nathan Zorndorf
 * Description: Uses DMA Controller 1 Channels 0-3 to take audio in from I2S2 RX, and pass it straight through to I2S2 TX
 * Edited by: Tyler Olivieri 
 */

#include <stdio.h>
#include <math.h> 
#include <tms320.h>
#include <Dsplib.h>
#include <my_types.h>
#include <usbstk5505.h>
#include <Application_1_Modified_Registers.h>
#include <Audio_Straight_Through_Using_DMA.h>

Int16 OverlapInL[WND_LEN];
Int16 OverlapInR[WND_LEN];
Int16 OverlapOutL[OVERLAP_LENGTH];
Int16 OverlapOutR[OVERLAP_LENGTH];

/* Buffers for processing */
#pragma DATA_SECTION(BufferL,"BufL");
Int16 BufferL[FFT_LENGTH];
#pragma DATA_SECTION(BufferR,"BufR");
Int16 BufferR[FFT_LENGTH];
#pragma DATA_SECTION(realL, "rfftL");
Int16 realL[FFT_LENGTH];
#pragma DATA_SECTION(realR, "rfftR");
Int16 realR[FFT_LENGTH];
/* -------------------------------------------*/

int Audio_Straight_Through_Using_DMA(void) {

	int i;
	int j = 0;
	ushort oflag = 0;
	
	/* Initialize buffers */
	for (i = 0; i < WND_LEN; i++) {
		OverlapInL[i] = 0;
		OverlapInR[i] = 0;
	}
	for (i = 0; i < OVERLAP_LENGTH; i++) {
		OverlapOutL[i] = 0;
		OverlapOutR[i] = 0;
	}
	
	/* Begin infinite loop */
	//while ( j < 30000)
	while(1) 
	{
		j++;
        /* Get new input audio block */
		if (PingPongFlagInL && PingPongFlagInR)  // Last Transfer complete was Pong - Filling Ping
		{
			// Copy previous NEW data to current OLD data
			memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);
		    memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);
		    
		    // Update NEW data with current audio in 
		    memmove(&OverlapInL[FRAME],&DMA_InpL[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE);
		    memmove(&OverlapInR[FRAME],&DMA_InpR[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE); 
		}
		else  									// Last Transfer complete was Ping - Filling Pong
		{
			// Copy previous NEW data to current OLD data
		    memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);
		    memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);	
		    
		    // Update NEW data with current audio in 
		    memmove(&OverlapInL[FRAME],&DMA_InpL[0], sizeof(Int16)*AUDIO_IO_SIZE);
		    memmove(&OverlapInR[FRAME],&DMA_InpR[0], sizeof(Int16)*AUDIO_IO_SIZE);  
		}
      
        //oflag = fir((DATA *)&OverlapInL,(DATA *)&filt_coefficients, (DATA *)&BufferL, WND_LEN, filt_Order);
        
        //straight through processing
        memmove(&BufferL[0],&OverlapInL[0], sizeof(Int16)*WND_LEN);
        memmove(&BufferR[0],&OverlapInR[0], sizeof(Int16)*WND_LEN);
        

        if (PingPongFlagOutL && PingPongFlagOutR) // Last Transfer complete was Pong - Filling Ping
        {
        	for (i = 0; i < OVERLAP_LENGTH; i++) 
        	{
        		
        		
        		/* Current output block is previous overlapped block + current processed block */
        		DMA_OutL[i + AUDIO_IO_SIZE] = BufferL[i];
        		DMA_OutR[i + AUDIO_IO_SIZE] = BufferR[i]; 

				/* Update overlap buffer */
        		OverlapOutL[i] = BufferL[i + FRAME];
        		OverlapOutR[i] = BufferR[i + FRAME];
        	}
        } 
        else 									// Last Transfer complete was Ping - Filling Pong
        {
        	
       
        	for (i = 0; i < OVERLAP_LENGTH; i++) 
        	{
		        
        		/* Current output block is previous overlapped block + current processed block */
        		DMA_OutL[i] =  BufferL[i];
        		DMA_OutR[i] =  BufferR[i]; 
        		/* Update overlap buffer */
        		OverlapOutL[i] = BufferL[i + FRAME];
        		OverlapOutR[i] = BufferR[i + FRAME];
        	}
        }
	}

}
