/*  ============================================================================
 *   Author: Nathan Zorndorf 
 *   Description: UART Initialization and Test 
 * Modified by Tyler Olivieri 
 */

#include <stdio.h>
#include <csl_general.h>
#include <csl_uart.h>
#include <My_PLL.h>
#include <usbstk5505.h>
#include <My_UART.h>
#include <Application_1_Modified_Registers.h>

// UART #define's
#define TIMEOUT_VALUE	0 //No timeout
#define COUNT 			2 //number of bytes to send over uart
#define ONE_HALF_SECOND	500000 // 500,000 microseconds in 0.5 seconds

CSL_UartObj 	uartObj; // Used to create the UART instance handle. 
CSL_UartHandle	hUart;	 // The UART instance handle. 

extern Int16 x[];
	
int My_UART(void){
	
	Int16 TxBuf = 20000;
	int j = 0;
	//int z = 0;
	Int16 status;  // For checking if functions ran successfully. 
	
	//Int16 y[3];
	//char serial_y[COUNT];
	//int size;
	//Int16 mask = 255; // 0000000011111111
	//char first_half;
	//char second_half;
	//char* test_char = "hello";
	Int16 RxBuf;
		CSL_UartSetup uartSetup =
		{
			// Input clock freq in MHz 
		    100000000,
			// Baud rate 
		    //230400,
		    115200,
			// Word length of 8 
		    CSL_UART_WORD8,
			// To generate 1 stop bit 
		    1, 
			// Disable the parity 
		    CSL_UART_DISABLE_PARITY,
		    //Enable FIFO buffer for UART
			CSL_UART_FIFO_DISABLE, // - FIFO buffer for the uart so that it does not send an interrupt , FIFO buffer is limited to 16  
			// to the cpu every time it wants to send a byte 
			// this should allow other processing to happen?
			// Loop Back enable 
		    CSL_UART_NO_LOOPBACK,
			// No auto flow control
			CSL_UART_NO_AFE , // - does not matter, choose OFF anyway.
			// No RTS 
			CSL_UART_NO_RTS , // - does not matter because AFE is OFF, choose NO_RST anyway. 
		};
	
	 
    // Initialize UART
	status = UART_init(&uartObj, CSL_UART_INST_0, UART_POLLED); 
    	if(CSL_SOK != status) { printf("UART_init failed error code %d\n",status); return(status); }
    	else { printf("UART_init Successful\n"); }
    	
	hUart = (CSL_UartHandle)&uartObj; //intsantiate specific UART instance

     
	// UART Setup
    status = UART_setup(hUart, &uartSetup);
    	if(CSL_SOK != status) { printf("UART_setup failed error code %d\n",status); return(status); }
    	else { printf("UART_setup Successful\n"); }
    
    // External Bus Selection Register 
	CPU_EBSR = 0x1000; // Mode 1 (SPI, GPIO, UART, and I2S2) - OK
	
	// Test outputs  
	//y[0] 	= 16961; //'BA'
	//y[1]	= 17475; //'DC'
	//y[2] 	= 18503; //'HG'
	   
	//status = UART_write(hUart, (Char*)x, COUNT/2, TIMEOUT_VALUE);
	//status = UART_write(hUart, serial_y, COUNT, TIMEOUT_VALUE);
	//printf("Sent note ON message viat UART.\n\n");

	//read and write from MATLAB
	//status = UART_write(hUart, (Char *) &TxBuf, COUNT/2, TIMEOUT_VALUE);
	if(CSL_SOK != status) { printf("UART_init failed error code %d\n",status); }
	
	
	//status =  UART_read(hUart, (Char *) &RxBuf, COUNT/2, TIMEOUT_VALUE);
	if(CSL_SOK != status) { printf("UART_init failed error code %d\n",status); return(status); }
	if(RxBuf != 10000)
	{
		printf("UART recieve failed\n");
	}
	printf("%d\n", RxBuf);
	printf("My_UART.c complete.\n");
	
	return(0);
	
}
