#ifndef DMA_DRIVER_H_
#define DMA_DRIVER_H_

#include "tistdtypes.h"


/* Setup a struct that contains all data that the user will enter to be used in the program 
   * It is defined as a typedef so that it can be declared as 
   * Dma_setup argument
   * */  
   
typedef enum dmaTransferMode{

	DARAM2DARAM = 0,
	sourceDARAM,
	destDARAM
}dmaTransferMode;
 
typedef struct  {
 	
   	Uint16 baseAddress;
   	dmaTransferMode mode;
 	
}Dma_Obj;
   

#define sourceStartL 0x0
#define sourceStartU 0x1
#define destStartL 0x2
#define destStartU 0x3
#define transferControl1 0x4
#define transferControl2 0x5
#define CH0 1
#define CH1 2
#define CH2 3
#define CH3 4
#define DARAMoffset 0x010000

#define ACCESS_REG(offset) *(ioport volatile unsigned *)(hDma->baseAddress + offset)


/* ==================== Some CPU Registers ==================== */
#define IER0        			 *(volatile unsigned *)0x0000
#define IFR0        			 *(volatile unsigned *)0x0001
#define IER1        			 *(volatile unsigned *)0x0045
#define IFR1        			 *(volatile unsigned *)0x0046
#define PRCR       				 *(volatile ioport Uint16*)(0x1C05)

/* ==================== General DMA Registers ==================== */
#define DMA_IFR     			 *(ioport volatile unsigned *)0x1C30    // DMA Interrupt Flag Register
#define DMA_IER     			 *(ioport volatile unsigned *)0x1C31    // DMA Interrupt Enable Register

 Int16 Dma_init(Dma_Obj * hDma,  Uint16 base_Address, dmaTransferMode mode);
 Int16 Dma_Config(Dma_Obj * hDma, Int16 * sourceAddress, Int16 * destAddress, Uint16 transferLength);
 Int16 Dma_start(Dma_Obj * hDma);
 Int16 Dma_interrupts();
 Int16 Dma_enablePingPong (Dma_Obj * hDma);
 Int16 Dma_enable_I2S();
 
#ifndef NULL
#define NULL            ((void*)0)
#endif



#endif /*DMA_DRIVER_H_*/
