/* ===============================================
 * Author: Nathan Zorndorf
 * Description: Uses DMA Controller 1 Channels 0-3 to take audio in from I2S2 RX, do processing and return the fundamental frequency of the input audio signal. 
 */

#include <stdio.h>
#include <math.h> 
#include <tms320.h>
#include <Dsplib.h>
#include <usbstk5505.h>
#include <Application_1_Modified_Registers.h>
#include <Audio_To_MIDI_Using_DMA_and_CFFT.h>
#include <hwafft.h>
#include "spectrumAnalyzer_DMA_cfft.h"
#include "outputFFT.h"
#include "block_FIR_uart.h"

Int16 OverlapInL[WND_LEN];
Int16 OverlapInR[WND_LEN];
Int16 OverlapOutL[OVERLAP_LENGTH];
Int16 OverlapOutR[OVERLAP_LENGTH];

/* --- buffers required for processing  ----*/
#pragma DATA_SECTION(BufferL,"BufL");
Int16 BufferL[FFT_LENGTH];
#pragma DATA_SECTION(BufferR,"BufR");
Int16 BufferR[FFT_LENGTH];
#pragma DATA_SECTION(realL, "rfftL");
long realL[FFT_LENGTH];
#pragma DATA_SECTION(realR, "rfftR");
long realR[FFT_LENGTH];
#pragma DATA_SECTION(imagL, "ifftL");
Int16 imagL[FFT_LENGTH];
#pragma DATA_SECTION(imagR, "ifftR");
Int16 imagR[FFT_LENGTH];
#pragma DATA_SECTION(PSD_Result, "PSD");
Int16 PSD_Result[FFT_LENGTH];
#pragma DATA_SECTION(PSD_Result_sqrt, "PSD_sqrt");
Int16 PSD_Result_sqrt[FFT_LENGTH];
/* -------------------------------------------*/
/* --- Special buffers required for CFFT ---*/
#pragma DATA_SECTION(complex_data,"cmplxBuf");
LDATA complex_data[2*FFT_LENGTH];
/* -------------------------------------------*/


int spectrumAnalyzer_DMA_cfft(void) {

	int i = 0;
    int counter = 0;
    
    //memset??
	/* Initialize buffers */
	for (i = 0; i < WND_LEN; i++) {
		OverlapInL[i] = 0;
		OverlapInR[i] = 0;
	}
	for (i = 0; i < OVERLAP_LENGTH; i++) {
		OverlapOutL[i] = 0;
		OverlapOutR[i] = 0;
	}
	
	// Begin infinite loop 
	
	while (1) 
	{
        // Get new input audio block 
		if (PingPongFlagInL && PingPongFlagInR)  // Last Transfer complete was Pong - Filling Ping
		{
			
		    // Copy previous NEW data to current OLD data
			memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);
		    memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);
		    
		    // Update NEW data with current audio in 
		    memmove(&OverlapInL[FRAME],&DMA_InpL[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE);
		    memmove(&OverlapInR[FRAME],&DMA_InpR[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE); 
		    
		}
		else  									// Last Transfer complete was Ping - Filling Pong
		{
			
		    // Copy previous NEW data to current OLD data
		    memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);
		    memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);	
		    
		    // Update NEW data with current audio in 
		    memmove(&OverlapInL[FRAME],&DMA_InpL[0], sizeof(Int16)*AUDIO_IO_SIZE);
		    memmove(&OverlapInR[FRAME],&DMA_InpR[0], sizeof(Int16)*AUDIO_IO_SIZE);  
		    
		}

		
	        
        //straight through processing
        memmove(&BufferL[0],&OverlapInL[0], sizeof(Int16)*WND_LEN);
        memmove(&BufferR[0],&OverlapInR[0], sizeof(Int16)*WND_LEN);
        
	
	
		// Convert real data to "pseudo"-complex data (real, 0) 
     	// Int32 complex = Int16 real (MSBs) + Int16 imag (LSBs) 
	
	/* //for FFT debugging
	while(1){		
		block_FIR_uart(PSD_Result, BufferR);
	*/
	    //Compute FFT
	    //Output data using UART
     	counter++;
		if (counter ==1000)
		{
	
			for (i = 0; i < FFT_LENGTH; i++) 
			{
				complex_data[2*i] = BufferR[i]; // place audio data (Real) in each even index of complex_data
			 	complex_data[2*i+1] = 0;		// place a 0 (Imag) in each odd index of complex_data
			}

			//move fft in for loop...... make counter bigger 	
			// Perform FFT 
			//cfft32(complex_data, HOP_SIZE, SCALE);
			
			cfft32_SCALE(complex_data, FFT_LENGTH); 

			// Perform bit-reversing 
			cbrev32(complex_data, complex_data, FFT_LENGTH);
	
			// Extract real and imaginary parts 
			for (i = 0; i < NUM_BINS; i++) {
				realR[i] = complex_data[2*i];
				imagR[i] = complex_data[2*i+1];
			

	
			// Find the Power of the audio signal using the cfft results and scale by 1/2 
			//for(i = 0; i < FFT_LENGTH; i++) { // square the real vector and the imaginary vector
				realR[i] = (realR[i] * realR[i]);	
				imagR[i] = (imagR[i] * imagR[i]);
			//}
		
			// Find the Power of the audio signal using the cfft results and scale by 1/2 
			//for(i = 0; i < FFT_LENGTH; i++) { // square the real vector and the imaginary vector
				//realR[i] = realR[i] >> 15;	
				//imagR[i] = imagR[i] >> 15;
			//}
		
			// ADD 
			//for(i = 0; i < FFT_LENGTH; i++) {
				PSD_Result[i] = realR[i] + imagR[i];
			}
		
			//square root to find magnitude 
			//logn((DATA *) PSD_Result, (LDATA *) PSD_Result_sqrt, FFT_LENGTH);
	   		 //sqrt_16((DATA*) PSD_Result, (DATA *) PSD_Result_sqrt, FFT_LENGTH);
			
            
			counter = 0;
			//outputFFT(PSD_Result);
	    }
	//for FFT debugging }
	
        if (PingPongFlagOutL && PingPongFlagOutR) // Last Transfer complete was Pong - Filling Ping
        {
        	for (i = 0; i < OVERLAP_LENGTH; i++) 
        	{
        		// Current output block is previous overlapped block + current processed block 
        		DMA_OutL[i + AUDIO_IO_SIZE] = OverlapOutL[i] + BufferL[i];
        		DMA_OutR[i + AUDIO_IO_SIZE] = OverlapOutR[i] + BufferR[i]; 

        	}	// Update overlap buffer 
			memmove(&OverlapOutL[0], &BufferL[FRAME], sizeof(Int16)*OVERLAP_LENGTH);
			memmove(&OverlapOutR[0], &BufferR[FRAME], sizeof(Int16)*OVERLAP_LENGTH);	
        } 
        else 									// Last Transfer complete was Ping - Filling Pong
        {
        	for (i = 0; i < OVERLAP_LENGTH; i++) 
        	{
        		// Current output block is previous overlapped block + current processed block 
        		DMA_OutL[i] = OverlapOutL[i] + BufferL[i];
        		DMA_OutR[i] = OverlapOutR[i] + BufferR[i]; 
 
        	}	// Update overlap buffer 
        	memmove(&OverlapOutL[0], &BufferL[FRAME], sizeof(Int16)*OVERLAP_LENGTH);
        	memmove(&OverlapOutR[0], &BufferR[FRAME], sizeof(Int16)*OVERLAP_LENGTH);	
        	
        }
	}

}


