
/* ===============================================
 * Author: Nathan Zorndorf
 * Description: Uses DMA Controller 1 Channels 0-3 to take audio in from I2S2 RX, do processing and return the fundamental frequency of the input audio signal. 
 */

#include <stdio.h>
#include <math.h> 
#include <tms320.h>
#include <Dsplib.h>
#include <usbstk5505.h>
#include <Application_1_Modified_Registers.h>
#include <Audio_To_MIDI_Using_DMA_and_CFFT.h>
#include <hwafft.h>
#include "spectrumAnalyzer_dma_hwfft.h"
#include "outputFFT.h"
#include "block_FIR_uart.h"


extern Int16 OverlapInL[WND_LEN];
extern Int16 OverlapInR[WND_LEN];
extern Int16 OverlapOutL[OVERLAP_LENGTH];
extern Int16 OverlapOutR[OVERLAP_LENGTH];
extern Int16 BufferL[FFT_LENGTH];
extern Int16 BufferR[FFT_LENGTH];

#pragma DATA_SECTION(realL, "rfftL");
long realL[FFT_LENGTH];
#pragma DATA_SECTION(realR, "rfftR");
long realR[FFT_LENGTH];
#pragma DATA_SECTION(imagL, "ifftL");
Int16 imagL[FFT_LENGTH];
#pragma DATA_SECTION(imagR, "ifftR");
Int16 imagR[FFT_LENGTH];
#pragma DATA_SECTION(PSD_Result, "PSD");
Int16 PSD_Result[FFT_LENGTH];
#pragma DATA_SECTION(PSD_Result_sqrt, "PSD_sqrt");
Int16 PSD_Result_sqrt[FFT_LENGTH];
/* --------------------------------
/* --- Special buffers required for HWAFFT ---*/
#pragma DATA_SECTION(complex_buffer, "cmplxBuf");
long complex_buffer[WND_LEN];

#pragma DATA_SECTION(bitreversed_buffer, "brBuf");
#pragma DATA_ALIGN(bitreversed_buffer, 2*FFT_LENGTH);
long bitreversed_buffer[FFT_LENGTH];

#pragma DATA_SECTION(temporary_buffer,"tmpBuf");
long temporary_buffer[FFT_LENGTH];
/* -------------------------------------------*/
/* --- buffers required for processing  ----*/
#pragma DATA_SECTION(BufferL,"BufL");
Int16 BufferL[FFT_LENGTH];
#pragma DATA_SECTION(BufferR,"BufR");
Int16 BufferR[FFT_LENGTH];
/* -------------------------------------------*/



int spectrumAnalyzer_DMA_hwfft(void) {

	int i = 0;
    int counter = 0;
	Uint16 data_selection;
	Int32 *complex_data, *bitrev_data, *temp_data, *fft_data;
		
    
    //memset??
	/* Initialize buffers */
	for (i = 0; i < WND_LEN; i++) {
		OverlapInL[i] = 0;
		OverlapInR[i] = 0;
	}
	for (i = 0; i < OVERLAP_LENGTH; i++) {
		OverlapOutL[i] = 0;
		OverlapOutR[i] = 0;
	}
	
	// Begin infinite loop 
/*	while (1) 
	{
     	   
        // Get new input audio block 
		if (PingPongFlagInL && PingPongFlagInR)  // Last Transfer complete was Pong - Filling Ping
		{
			
		    // Copy previous NEW data to current OLD data
			memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);
		    memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);
		    
		    // Update NEW data with current audio in 
		    memmove(&OverlapInL[FRAME],&DMA_InpL[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE);
		    memmove(&OverlapInR[FRAME],&DMA_InpR[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE); 
		    
		}
		else  									// Last Transfer complete was Ping - Filling Pong
		{
			
		    // Copy previous NEW data to current OLD data
		    memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);
		    memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);	
		    
		    // Update NEW data with current audio in 
		    memmove(&OverlapInL[FRAME],&DMA_InpL[0], sizeof(Int16)*AUDIO_IO_SIZE);
		    memmove(&OverlapInR[FRAME],&DMA_InpR[0], sizeof(Int16)*AUDIO_IO_SIZE);  
		    
		}

		
	        
        //straight through processing
        memmove(&BufferL[0],&OverlapInL[0], sizeof(Int16)*WND_LEN);
        memmove(&BufferR[0],&OverlapInR[0], sizeof(Int16)*WND_LEN);
        
	*/
	
		// Convert real data to "pseudo"-complex data (real, 0) 
     	// Int32 complex = Int16 real (MSBs) + Int16 imag (LSBs) 
	
	//for FFT debugging
	while(1){		
		block_FIR_uart(PSD_Result, BufferR);
	
	    //Compute FFT
	    //Output data using UART
     	counter++;
		if (counter ==1)
		{
			/* Convert real data to "pseudo"-complex data (real, 0) */
			/* Int32 complex = Int16 real (MSBs) + Int16 imag (LSBs) */
			for (i = 0; i < FFT_LENGTH; i++) 
			{
				complex_buffer[i] = ((long) BufferR[i]) << 16;
			}
			/* Perform bit-reversing */
			hwafft_br((Int32*)complex_buffer, (Int32*)bitreversed_buffer, FFT_LENGTH);
		
			 // hwafft_#pts, where # = 2*FRAME 
			data_selection = hwafft_512pts((Int32*)bitreversed_buffer, (Int32*)temporary_buffer, FFT_FLAG, SCALE_FLAG); 
			
		    //data_selection = hwafft_512pts((Int32*)bitreversed_buffer, (Int32*)temporary_buffer, FFT_FLAG, NOSCALE_FLAG);
			/* Extract real and imaginary parts 
			 * Square the real and imaginary vector
			 * shift so there is no overflow
			 * add together for the sum of squares
			 * if possible, sqrt 
			 * 
			 * */
			if (data_selection == 1) 
			{
				//fft_data = temp_data;	// results stored in this scratch vector 
				for (i = 0; i < FFT_LENGTH; i++) 
				{
					realR[i] = (Int16)(temporary_buffer[i] >> 16);
					imagR[i] = (Int16)(temporary_buffer[i] & 0x0000FFFF);
					realR[i] = (realR[i] * realR[i]>> 15);	
					imagR[i] = (imagR[i] * imagR[i] >> 15);
				    PSD_Result[i] = realR[i] + imagR[i];
				}
			} 
			else 
			{
				//fft_data = bitrev_data; // results stored in this data vector 
				for (i = 0; i < NUM_BINS; i++) 
				{
					realR[i] = (Int16)(bitreversed_buffer[i] >> 16);
					imagR[i] = (Int16)(bitreversed_buffer[i] & 0x0000FFFF);
					realR[i] = (realR[i] * realR[i]>>15);	
					imagR[i] = (imagR[i] * imagR[i]>>15);
				    PSD_Result[i] = realR[i] + imagR[i];
				}
			}
			//Reset counter and Output FFT
			counter = 0;
			outputFFT(PSD_Result);
	    }
	 }
	
      /*  if (PingPongFlagOutL && PingPongFlagOutR) // Last Transfer complete was Pong - Filling Ping
        {
        	for (i = 0; i < OVERLAP_LENGTH; i++) 
        	{
        		// Current output block is previous overlapped block + current processed block 
        		DMA_OutL[i + AUDIO_IO_SIZE] = OverlapOutL[i] + BufferL[i];
        		DMA_OutR[i + AUDIO_IO_SIZE] = OverlapOutR[i] + BufferR[i]; 
        	}
			// Update overlap buffer 
			memmove(&OverlapOutL[0], &BufferL[FRAME], sizeof(Int16)*OVERLAP_LENGTH);
			memmove(&OverlapOutR[0], &BufferR[FRAME], sizeof(Int16)*OVERLAP_LENGTH);	
        } 
        else 									// Last Transfer complete was Ping - Filling Pong
        {
        	for (i = 0; i < OVERLAP_LENGTH; i++) 
        	{
        		// Current output block is previous overlapped block + current processed block 
        		DMA_OutL[i] = OverlapOutL[i] + BufferL[i];
        		DMA_OutR[i] = OverlapOutR[i] + BufferR[i]; 
        	}
        	// Update overlap buffer 
        	memmove(&OverlapOutL[0], &BufferL[FRAME], sizeof(Int16)*OVERLAP_LENGTH);
        	memmove(&OverlapOutR[0], &BufferR[FRAME], sizeof(Int16)*OVERLAP_LENGTH);	
        }
	}*/

}


